from peewee import MySQLDatabase
import wrapt
from ..credentials import get_credentials

factdata_db = MySQLDatabase(None)


def connect_database(config=None):
    if config is None:
        config = get_credentials()['database']
    factdata_db.init(**config)
    factdata_db.connect()



import matplotlib.pyplot as plt
from matplotlib.dates import DateFormatter
import numpy as np


def plot_excess_rate(binned_runs, outputfile=None):
    '''
    Create an excess rate plot from given data

    Parameters
    ----------
    binned_runs: pd.DataFrame
        Binned data as returned by `fact.analysis.bin_runs`
    outputfile: path
        if not None, the plot is saved to this file

    Returns
    -------
    ax_excess: matplotlib.axes.Axes
        the matplotlib Axes for the excess rate plot
    ax_significance: matplotlib.axes.Axes
        the matplotlib Axes for the significance plot
    '''
    fig = plt.figure()

    ax_sig = plt.subplot2grid((8, 1), (6, 0), rowspan=2)
    ax_rate = plt.subplot2grid((8, 1), (1, 0), rowspan=5, sharex=ax_sig)

    colors = [e['color'] for e in plt.rcParams['axes.prop_cycle']]

    labels = []
    plots = []
    for (name, group), color in zip(binned_runs.groupby('source'), colors):
        if len(group.index) == 0:
            continue

        labels.append(name)
        plots.append(ax_rate.errorbar(
            x=group.time_mean.values,
            y=group.excess_rate_per_h.values,
            xerr=group.time_width.values / 2,
            yerr=group.excess_rate_err.values,
            label=name,
            linestyle='',
            mec='none',
            color=color,
        ))

        ax_sig.errorbar(
            x=group.time_mean.values,
            y=group.significance.values,
            xerr=group.time_width.values / 2,
            label=name,
            linestyle='',
            mec='none',
            color=color,
        )

    fig.legend(
        plots,
        labels,
        loc='upper center',
        ncol=3,
        columnspacing=0.5,
        numpoints=1,
        handletextpad=0.1,
        bbox_to_anchor=[0.5, 0.99],
    )

    ax_rate.set_ylabel('Excess Event Rate / $\mathrm{h}^{-1}$')
    ax_sig.set_ylabel('$S_{\mathrm{Li/Ma}} \,\, / \,\, \sigma$')

    ymax = max(3.25, np.ceil(ax_sig.get_ylim()[1]))
    ax_sig.set_ylim(0, ymax)
    ax_sig.set_yticks(np.arange(0, ymax + 0.1, ymax // 4 + 1))

    plt.setp(ax_rate.get_xticklabels(), visible=False)
    plt.setp(ax_sig.get_xticklabels(), rotation=30, va='top', ha='right')

    if binned_runs.night.nunique() <= 7:
        ax_sig.xaxis.set_major_formatter(DateFormatter('%Y-%m-%d %H:%M'))
    else:
        ax_sig.xaxis.set_major_formatter(DateFormatter('%Y-%m-%d'))

    fig.tight_layout()

    if outputfile is not None:
        fig.savefig(outputfile)

    return ax_rate, ax_sig



import pkg_resources as res
import numpy as np
from functools import lru_cache
import pandas as pd

from .constants import (
    FOCAL_LENGTH_MM, PINCUSHION_DISTORTION_SLOPE,
    PIXEL_SPACING_IN_MM, FOV_PER_PIXEL_DEG
)


def camera_distance_mm_to_deg(distance_mm):
    '''
    Transform a distance in mm in the camera plane
    to it's approximate equivalent in degrees.
    '''

    return distance_mm * FOV_PER_PIXEL_DEG / PIXEL_SPACING_IN_MM


pixel_mapping = np.genfromtxt(
    res.resource_filename('fact', 'resources/FACTmap111030.txt'),
    names=[
        'softID', 'hardID', 'geom_i', 'geom_j',
        'G-APD', 'V_op', 'HV_B', 'HV_C',
        'pos_X', 'pos_Y', 'radius'
    ],
    dtype=None,
)

non_standard_pixel_chids = dict(
    dead=[927, 80, 873],
    crazy=[863, 297, 868],
    twins=[            # the signals of these pairs of pixels are the same
        (1093, 1094),
        (527, 528),
        (721, 722),
    ]
)


GEOM_2_SOFTID = {
    (i, j): soft for i, j, soft in zip(
        pixel_mapping['geom_i'], pixel_mapping['geom_j'], pixel_mapping['softID']
    )}


def reorder_softid2chid(array):
    '''
    Returns view to the given array, remapped from softid order
    to chid order (e.g. MARS ordering to fact-tools ordering)
    '''
    return array[CHID_2_SOFTID]


@lru_cache(maxsize=1)
def get_pixel_dataframe():
    ''' return pixel mapping as pd.DataFrame

    '''
    pm = pd.DataFrame(pixel_mapping)
    pm.sort_values('hardID', inplace=True)
    # after sorting the CHID is in principle the index
    # of pm, but I'll like to have it explicitely
    pm['CHID'] = np.arange(len(pm))

    pm['trigger_patch_id'] = pm['CHID'] // 9
    pm['bias_patch_id'] = pm['HV_B'] * 32 + pm['HV_C']

    bias_patch_sizes = pm.bias_patch_id.value_counts().sort_index()
    pm['bias_patch_size'] = bias_patch_sizes[pm.bias_patch_id].values

    pm['x'] = -pm.pos_Y.values * PIXEL_SPACING_IN_MM
    pm['y'] = pm.pos_X.values * PIXEL_SPACING_IN_MM

    pm['x_angle'] = np.rad2deg(
        np.arctan(pm.x / FOCAL_LENGTH_MM) * 
        (1 + PINCUSHION_DISTORTION_SLOPE)
    )
    pm['y_angle'] = np.rad2deg(
        np.arctan(pm.y / FOCAL_LENGTH_MM) * 
        (1 + PINCUSHION_DISTORTION_SLOPE)
    )

    return pm


FOV_RADIUS = np.hypot(
    get_pixel_dataframe().x_angle, get_pixel_dataframe().y_angle
).max()


patch_indices = get_pixel_dataframe()[[
        'trigger_patch_id',
        'bias_patch_id',
        'bias_patch_size',
    ]].drop_duplicates().reset_index(drop=True)


@np.vectorize
def geom2soft(i, j):
    return GEOM_2_SOFTID[(i, j)]


def softid2chid(softid):
    return hardid2chid(pixel_mapping['hardID'])[softid]


def softid2hardid(softid):
    return pixel_mapping['hardID'][softid]


def hardid2chid(hardid):
    crate = hardid // 1000
    board = (hardid // 100) % 10
    patch = (hardid // 10) % 10
    pixel = (hardid % 10)
    return pixel + 9 * patch + 36 * board + 360 * crate


CHID_2_SOFTID = np.empty(1440, dtype=int)
for softid in range(1440):
    hardid = pixel_mapping['hardID'][softid]
    chid = hardid2chid(hardid)
    CHID_2_SOFTID[chid] = softid


def chid2softid(chid):
    return CHID_2_SOFTID[chid]


def hardid2softid(hardid):
    return chid2softid(hardid2chid(hardid))


def get_pixel_coords():
    '''
    Calculate the pixel coordinates from the standard pixel-map file
    by default it gets rotated by 90 degrees clockwise to show the same
    orientation as MARS and fact-tools
    '''
    df = get_pixel_dataframe()

    return df.x.values, df.y.values


@lru_cache(maxsize=1)
def bias_to_trigger_patch_map():
    by_chid = pixel_mapping['hardID'].argsort()

    bias_channel = pixel_mapping[by_chid]['HV_B'] * 32 + pixel_mapping[by_chid]['HV_C']

    _, idx = np.unique(bias_channel, return_index=True)

    return bias_channel[np.sort(idx)]


def combine_bias_patch_current_to_trigger_patch_current(bias_patch_currents):
    """
    For this to work, you need to know that the calibrated currents in FACT
    which are delivered by the program FEEDBACK for all 320 bias patches
    are given as "uA per pixel per bias patch", not as "uA per bias patch"
    So if you want to combine these currents into one value given as:
    "uA per trigger patch" or "uA per pixel per trigger patch", you need to know
    the number of pixels in a bias patch.

    Luckily this is given by our patch_indices() DataFrame
    """

    pi = patch_indices()
    fourers = pi[pi.bias_patch_size == 4].sort_values('trigger_patch_id')
    fivers = pi[pi.bias_patch_size == 5].sort_values('trigger_patch_id')

    b_c = bias_patch_currents  # just to shorten the name
    t_c = b_c[fourers.bias_patch_id.values] * 4/9 + b_c[fivers.bias_patch_id.values] * 5/9

    trigger_patch_currents = t_c  # unshorten the name
    return trigger_patch_currents


def take_apart_trigger_values_for_bias_patches(trigger_rates):
    """
    Assumption is you have 160 values from trigger patches,
    such as the trigger rates per patch, or the trigger threshold per patch
    or whatever else you cant find per patch.

    And for some reason you say: Well this is valid for the entire trigger
    patch really, but I believe it also correlates with the bias patch
    """
    pi = patch_indices().sort_values('bias_patch_id')

    return trigger_rates[pi.trigger_patch_id.values]



''' some functions to deal with FACT modified modified julian date

The time used most of the time in FACT is the number of days since 01.01.1970

So this time is related to unix time, since it has the same offset
(unix time is the number of seconds since 01.01.1970
(what time? noon? midnight??))
but it is also related to 'the' Modified Julian Date (MJD),
which is used by astronomers
in the sense, that it also counts days.

According to http://en.wikipedia.org/wiki/Julian_day,
there is quite a large number of
somehow modified julian dates, of which the MJD is only one.
So it might be okay, to introduce a new modification,
going by the name of FACT Julain Date (FJD).
'''
from __future__ import print_function, division
import time
import calendar
from datetime import datetime, timedelta
import logging

import dateutil
import dateutil.parser

import pandas as pd

OFFSET = (datetime(1970, 1, 1) - datetime(1, 1, 1)).days


def fjd(datetime_inst):
    ''' convert datetime instance to FJD
    '''
    if datetime_inst.tzinfo is None:
        logging.warning('datetime instance is not aware of its timezone.'
                        'result possibly wrong!')
    return calendar.timegm(datetime_inst.utctimetuple()) / (24. * 3600.)


def iso2dt(iso_time_string):
    ''' parse ISO time string to timezone aware datetime instance

    example
    2015-01-23T08:08+01:00
    '''
    datetime_inst = dateutil.parser.parse(iso_time_string)
    # make aware at any cost!
    if datetime_inst.tzinfo is None:
        logging.info('ISO time string did not contain timezone info. I assume UTC!')
        datetime_inst = datetime_inst.replace(tzinfo=dateutil.tz.tzutc())
    return datetime_inst


def run2dt(run_string):
    ''' parse typical FACT run file path string to datetime instance (UTC)

    example
    first you do this:

    '/path/to/file/20141231.more_text' --> '20141231'
    then call
    run2dt('20141231')
    '''
    format_ = '%Y%m%d'
    datetime_inst = datetime.strptime(run_string, format_)
    datetime_inst = datetime_inst.replace(tzinfo=dateutil.tz.tzutc())
    return datetime_inst


def facttime(time_string):
    ''' conver time-string to fact time
    '''
    return calendar.timegm(time.strptime(
        time_string, '%Y%m%d %H:%M')) / (24. * 3600.)


def to_datetime(fact_julian_date):
    ''' convert facttime to datetime instance
    '''
    unix_time = fact_julian_date * 24 * 3600
    datetime_inst = datetime.utcfromtimestamp(unix_time)
    return datetime_inst


def datestr(datetime_inst):
    ''' make iso time string from datetime instance
    '''
    return datetime_inst.isoformat('T')


def night(timestamp=None):
    '''
    gives the date for a day change at noon instead of midnight
    '''
    if timestamp is None:
        timestamp = datetime.utcnow()
    if timestamp.hour < 12:
        timestamp = timestamp - timedelta(days=1)
    return timestamp


def night_integer(date):
    ''' return FACT night integer for date
    '''
    date -= pd.Timedelta(days=0.5)
    night_int = date.year * 10000 + date.month * 100 + date.day
    return night_int



from os import path
import pandas as pd
import json
import h5py
import sys
import logging
import numpy as np
from copy import copy


__all__ = [
    'write_data',
    'to_native_byteorder',
    'read_data',
    'read_h5py',
    'read_h5py_chunked',
    'check_extension',
    'to_h5py',
]

log = logging.getLogger(__name__)


allowed_extensions = ('.hdf', '.hdf5', '.h5', '.json', '.jsonl', '.jsonlines', '.csv')
native_byteorder = native_byteorder = {'little': '<', 'big': '>'}[sys.byteorder]


def write_data(df, file_path, key='data', use_hp5y=False, **kwargs):

    name, extension = path.splitext(file_path)

    if extension in ['.hdf', '.hdf5', '.h5']:
        if use_hp5y is True:
            to_h5py(file_path, df, key=key, **kwargs)
        else:
            df.to_hdf(file_path, key=key, **kwargs)

    elif extension == '.json':
        df.to_json(file_path, **kwargs)

    elif extension in ('.jsonl', '.jsonline'):
        df.to_json(file_path, lines=True, orient='records', **kwargs)

    elif extension == '.csv':
        index = kwargs.pop('index', False)
        df.to_csv(file_path, index=index, **kwargs)

    else:
        raise IOError(
            'cannot write tabular data with format {}. Allowed formats: {}'.format(
                extension, allowed_extensions,
            )
        )


def to_native_byteorder(array):
    ''' Convert numpy array to native byteorder '''

    # '|' : not-applicable, '=': native
    if array.dtype.byteorder not in ('|', '=', native_byteorder):
        return array.byteswap().newbyteorder()

    return array


def read_h5py(file_path, key='data', columns=None, mode='r+'):
    '''
    Read a hdf5 file written with h5py into a dataframe

    Parameters
    ----------
    file_path: str
        file to read in
    key: str
        name of the hdf5 group to read in
    columns: iterable[str]
        Names of the datasets to read in. If not given read all 1d datasets
    '''
    with h5py.File(file_path, mode) as f:
        group = f.get(key)
        if group is None:
            raise IOError('File does not contain group "{}"'.format(key))

        # get all columns of which don't have more than one value per row
        if columns is None:
            columns = [col for col in group.keys() if group[col].ndim == 1]

        df = pd.DataFrame()
        for col in columns:
            array = to_native_byteorder(group[col][:])

            # pandas cannot handle bytes, convert to str
            if array.dtype.kind == 'S':
                array = array.astype(str)

            if array.ndim == 1:
                df[col] = array
            elif array.ndim == 2:
                for i in range(array.shape[1]):
                    df[col + '_{}'.format(i)] = array[:, i]
            else:
                log.warning('Skipping column {}, not 1d or 2d'.format(col))

        if 'index' in df.columns:
            df.set_index('index', inplace=True)

    return df


def h5py_get_n_rows(file_path, key='data', mode='r+'):

    with h5py.File(file_path, mode) as f:
        group = f.get(key)

        if group is None:
            raise IOError('File does not contain group "{}"'.format(key))

        return group[next(iter(group.keys()))].shape[0]


def read_h5py_chunked(file_path, key='data', columns=None, chunksize=None, mode='r+'):
    '''
    Generator function to read from h5py hdf5 in chunks,
    returns an iterator over pandas dataframes.

    When chunksize is None, use 1 chunk
    '''
    with h5py.File(file_path, mode) as f:
        group = f.get(key)
        if group is None:
            raise IOError('File does not contain group "{}"'.format(key))

        # get all columns of which don't have more than one value per row
        if columns is None:
            columns = [col for col in group.keys() if group[col].ndim == 1]

        n_rows = h5py_get_n_rows(file_path, key=key)

        if chunksize is None:
            n_chunks = 1
            chunksize = n_rows
        else:
            n_chunks = int(np.ceil(n_rows / chunksize))
            log.info('Splitting data into {} chunks'.format(n_chunks))

        for col in copy(columns):
            if group[col].ndim > 2:
                columns.remove(col)
                log.warning('Ignoring column {}, not 1d or 2d'.format(col))

        for chunk in range(n_chunks):

            start = chunk * chunksize
            end = min(n_rows, (chunk + 1) * chunksize)

            df = pd.DataFrame(index=np.arange(start, end))

            for col in columns:
                array = to_native_byteorder(group[col][start:end])

                # pandas cannot handle bytes, convert to str
                if array.dtype.kind == 'S':
                    array = array.astype(str)

                if array.ndim == 1:
                    df[col] = array

                else:
                    for i in range(array.shape[1]):
                        df[col + '_{}'.format(i)] = array[:, i]

            yield df, start, end


def read_data(file_path, **kwargs):
    '''
    This is a utility wrapper for other reading functions.
    It will look for the file extension and try to use the correct
    reader and return a dataframe.

    Currently supported are hdf5 (pandas and h5py), json, jsonlines and csv

    Parameters
    ----------
    file_path: str
        Path to the input file

    All kwargs are passed to the individual reading function:

    pandas hdf5:   pd.read_hdf
    h5py hdf5:     fact.io.read_h5py
    json:          pd.DataFrame(json.load(file))
    jsonlines:     pd.read_json
    csv:           pd.read_csv
    '''
    name, extension = path.splitext(file_path)

    if extension in ['.hdf', '.hdf5', '.h5']:
        try:
            df = pd.read_hdf(file_path, **kwargs)
        except (TypeError, ValueError):
            df = read_h5py(file_path, **kwargs)

    elif extension == '.json':
        with open(file_path, 'r') as j:
            d = json.load(j)
            df = pd.DataFrame(d)

    elif extension in ('.jsonl', '.jsonlines'):
        df = pd.read_json(file_path, lines=True, **kwargs)

    elif extension == '.csv':
        df = pd.read_csv(file_path, **kwargs)

    else:
        raise NotImplementedError('Unknown data file extension {}'.format(extension))

    return df


def check_extension(file_path, allowed_extensions=allowed_extensions):
    p, extension = path.splitext(file_path)
    if extension not in allowed_extensions:
        raise IOError('Allowed formats: {}'.format(allowed_extensions))


def to_h5py(filename, df, key='data', mode='a', dtypes=None, index=True, **kwargs):
    '''
    Write pandas dataframe to h5py style hdf5 file

    Parameters
    ----------
    filename: str
        output file name
    df: pd.DataFrame
        The data to write out
    key: str
        the name for the hdf5 group to hold all datasets, default: data
    mode: str
        'w' to overwrite existing files, 'a' to append
    dtypes: dict
        if given, a mapping of column names to dtypes for conversion.
    index: bool
        If bool, also save the index of the dataframe

    All `**kwargs` are passed to h5py.create_dataset
    '''
    assert mode in ('w', 'a'), 'mode has to be either "a" or "w"'

    array = df.to_records(index=index)

    if dtypes is not None:
        array = change_recarray_dtype(array, dtypes)

    with h5py.File(filename, mode=mode) as f:
        if key not in f:
            initialize_h5py(f, array.dtype, key=key, **kwargs)

        append_to_h5py(f, array, key=key)


def change_recarray_dtype(array, dtypes):
    '''
    Change dtypes of recarray columns

    Parameters
    ----------
    array: np.recarray
      The array to modify
    dtypes: dict
        if given, a mapping of column names to dtypes for conversion.

    Returns
    -------
    array: np.recarray
        Copy of the original array with changed dtypes
    '''

    # get a modifieabale list of dtypes
    dt = array.dtype.descr
    # create a mapping from column name to index
    idx = {k: i for i, (k, d) in enumerate(dt)}

    for col, new_type in dtypes.items():
        dt[idx[col]] = (col, new_type)

    dt = np.dtype(dt)

    return array.astype(dt)


def initialize_h5py(f, dtypes, key='events', **kwargs):
    '''
    Create a group with name `key` and empty datasets for each
    entry in dtypes.

    Parameters
    ----------
    f: h5py.File
        the hdf5 file, opened either in write or append mode
    dtypes: numpy.dtype
        the numpy dtype object of a record or structured array describing
        the columns
    key: str
        the name for the hdf5 group to hold all datasets, default: data

    All kwargs are passed to h5py create_dataset
    '''
    group = f.create_group(key)

    for name in dtypes.names:
        dtype = dtypes[name]
        maxshape = [None] + list(dtype.shape)
        shape = [0] + list(dtype.shape)

        if dtype.base == object:
            dt = h5py.special_dtype(vlen=str)

        elif dtype.type == np.datetime64:
            # save dates as ISO string, create dummy date to get correct length
            dt = np.array(0, dtype=dtype).astype('S').dtype

        else:
            dt = dtype.base

        group.create_dataset(
            name,
            shape=tuple(shape),
            maxshape=tuple(maxshape),
            dtype=dt,
            **kwargs
        )

    return group


def append_to_h5py(f, array, key='events'):
    '''
    Append a numpy record or structured array to the given hdf5 file
    The file should have been previously initialized with initialize_hdf5

    Parameters
    ----------
    f: h5py.File
        the hdf5 file, opened either in write or append mode
    array: numpy.array or numpy.recarray
        the numpy array to append
    key: str
        the name for the hdf5 group with the corresponding data sets
    '''

    group = f.get(key)

    for column in array.dtype.names:
        dataset = group.get(column)

        n_existing_rows = dataset.shape[0]
        n_new_rows = array[column].shape[0]

        dataset.resize(n_existing_rows + n_new_rows, axis=0)

        # swap byteorder if not native
        if array[column].dtype.byteorder not in ('=', native_byteorder, '|'):
            data = array[column].newbyteorder().byteswap()
        else:
            data = array[column]

        if data.dtype.type == np.datetime64:
            data = data.astype('S')

        if data.ndim == 1:
            dataset[n_existing_rows:] = data

        elif data.ndim == 2:
            dataset[n_existing_rows:, :] = data

        else:
            raise NotImplementedError('Only 1d and 2d arrays are supported at this point')



import matplotlib
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from mpl_toolkits.axes_grid1 import make_axes_locatable
from matplotlib.figure import Figure
import numpy as np
import os
from .utils import calc_linewidth
from ..instrument import get_pixel_coords
from . import camera

import tkinter as tk
from tkinter import filedialog


class Viewer():
    """
    A tkinter based GUI to look at fact events in the camera view.

    Attributes
    ----------

    dataset  : array like with shape (num_events, 1440)
        the data you want to plot into the pixels
    label    : str
        the label for the colormap
    clickedcolour: a matplotlib conform colour represantation
        the coulour for clicked pixel
        [default: red]
    mapfile : str
        path/to/fact/pixelmap.csv
        [default pixel-map.csv]
    cmap : str or matplotlib colormap instance
        the colormap to use for plotting the 'dataset'
        [default: gray]
    vmin : float
        the minimum for the colorbar, if None min(dataset[event]) is used
        [default: None]
    vmax : float
        the maximum for the colorbar, if None max(dataset[event]) is used
        [default: None]
    """
    def __init__(self,
                 dataset,
                 label,
                 clickedcolour="r",
                 mapfile="pixel-map.csv",
                 cmap="gray",
                 vmin=None,
                 vmax=None,
                 ):
        matplotlib.use('TkAgg', warn=False, force=True)
        matplotlib.rcdefaults()
        self.event = 0
        if dataset.shape == (1440, ):
            self.dataset = np.reshape(dataset, (1, 1440))
        elif dataset.shape[1] == 1440:
            self.dataset = dataset
        else:
            raise ValueError('Viewer expects dataset with shape (1440, )\n'
                             'or (n_events, 1440)'
                             )
        self.numEvents = dataset.shape[0]
        self.clickedcolour = clickedcolour
        self.label = label
        self.cmap = cmap
        self.vmin = vmin
        self.vmax = vmax
        self.pixel_x, self.pixel_y = get_pixel_coords()
        self.fig = Figure(figsize=(7, 6), dpi=100)

        self.init_plot()

        # ---- GUI Stuff ----

        self.root = tk.Tk()
        self.root.geometry(("1024x768"))
        self.root.wm_title("PyFactViewer")

        buttonFrame = tk.Frame(self.root)
        plotFrame = tk.Frame(self.root)
        infoFrame = tk.Frame(plotFrame)

        buttonFrame.pack(side=tk.TOP)
        plotFrame.pack(side=tk.BOTTOM, expand=True, fill=tk.BOTH)
        infoFrame.pack(side=tk.BOTTOM)

        self.canvas = FigureCanvasTkAgg(self.fig, master=plotFrame)
        self.canvas.mpl_connect("pick_event", self.onpick)
        self.canvas.mpl_connect("resize_event", self.redraw)
        self.canvas.show()
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

        self.canvas._tkcanvas.pack(side=tk.BOTTOM, fill=tk.BOTH, expand=1)
        self.root.bind('<Key>', self.on_key_event)

        self.quit_button = tk.Button(master=buttonFrame,
                                     text='Quit',
                                     command=self.quit,
                                     expand=None,
                                     )
        self.next_button = tk.Button(master=buttonFrame,
                                     text='Next',
                                     command=self.next,
                                     expand=None,
                                     )
        self.previous_button = tk.Button(master=buttonFrame,
                                         text='Previous',
                                         command=self.previous,
                                         expand=None,
                                         )
        self.save_button = tk.Button(master=buttonFrame,
                                     text='Save Image',
                                     command=self.save,
                                     expand=None,
                                     )

        self.eventstring = tk.StringVar()
        self.eventstring.set("EventNum: {:05d}".format(self.event))
        self.eventbox = tk.Label(master=buttonFrame,
                                 textvariable=self.eventstring,
                                 )

        self.eventbox.pack(side=tk.LEFT)
        self.previous_button.pack(side=tk.LEFT)
        self.next_button.pack(side=tk.LEFT)
        self.quit_button.pack(side=tk.RIGHT)
        self.save_button.pack(side=tk.RIGHT)

        self.infotext = tk.StringVar()
        self.infotext.set("Click on a Pixel")
        self.infobox = tk.Label(master=infoFrame, textvariable=self.infotext)
        self.infobox.pack(side=tk.LEFT)

        self.update()
        tk.mainloop()

    def init_plot(self):
        self.width, self.height = self.fig.get_figwidth(), self.fig.get_figheight()
        self.fig.clf()
        self.ax = self.fig.add_subplot(1, 1, 1, aspect=1)
        divider = make_axes_locatable(self.ax)
        self.cax = divider.append_axes("right", size="5%", pad=0.1)
        self.ax.set_axis_off()

        if self.vmin is None:
            vmin = np.min(self.dataset[self.event])
        else:
            vmin = self.vmin
        if self.vmax is None:
            vmax = np.max(self.dataset[self.event])
        else:
            vmax = self.vmax

        self.plot = camera(
            ax=self.ax,
            data=self.dataset[self.event],
            pixelcoords=None,
            cmap=self.cmap,
            vmin=vmin,
            vmax=vmax,
            linewidth=None,
            picker=False,
        )
        self.plot.set_picker(0)
        self.clicked_pixel = None

        self.cb = self.fig.colorbar(self.plot, cax=self.cax, label=self.label)
        self.cb.set_clim(vmin=vmin, vmax=vmax)
        self.cb.draw_all()

    def save(self):
        filename = filedialog.asksaveasfilename(
            initialdir=os.getcwd(),
            parent=self.root,
            title="Choose a filename for the saved image",
            defaultextension=".pdf",
        )
        if filename:
            fig = self.fig
            fig.savefig(filename, dpi=300, bbox_inches="tight", transparent=True)
            print("Image sucessfully saved to", filename)

    def redraw(self, event):
        self.linewidth = calc_linewidth(self.ax)
        self.fig.tight_layout(pad=0)
        self.canvas.draw()

    def quit(self):
        self.root.quit()     # stops mainloop
        self.root.destroy()  # this is necessary on Windows to prevent

    def next(self):
        self.event = (self.event + 1) % len(self.dataset)
        self.update()

    def previous(self):
        if self.event == 0:
            self.event = self.numEvents - 1
        else:
            self.event -= 1
        self.update()

    def on_key_event(self, event):
        if event.keysym == "Right":
            self.next()
        elif event.keysym == "Left":
            self.previous()
        elif event.keysym == "q":
            self.quit()

    def update(self):
        self.plot.set_array(self.dataset[self.event])
        self.plot.changed()
        if self.vmin is None:
            vmin = np.min(self.dataset[self.event])
        else:
            vmin = self.vmin
        if self.vmax is None:
            vmax = np.max(self.dataset[self.event])
        else:
            vmax = self.vmax
        self.linewidth = calc_linewidth(self.ax)
        self.plot.set_linewidths(self.linewidth)
        self.cb.set_clim(vmin=vmin, vmax=vmax)
        self.cb.draw_all()
        self.plot.set_picker(0)
        self.canvas.draw()
        self.eventstring.set("EventNum: {:05d}".format(self.event))

    def onpick(self, event):
        hitpixel = event.ind[0]
        if hitpixel != self.clicked_pixel:
            self.clicked_pixel = hitpixel
            self.update()
            self.infotext.set(
                "chid: {:04d}, {} = {:4.2f}".format(
                    hitpixel,
                    self.label,
                    self.plot.get_array()[hitpixel]
                )
            )



import numpy as np


def li_ma_significance(n_on, n_off, alpha=0.2):
    '''
    Calculate the Li&Ma significance for given
    observations data

    Parameters
    ----------
    n_on: integer or array like
        Number of events for the on observations
    n_off: integer of array like
        Number of events for the off observations
    alpha: float
        Scaling factor for the off observations, for wobble observations
        this is 1 / number of off regions
    '''

    scalar = np.isscalar(n_on)

    n_on = np.array(n_on, copy=False, ndmin=1)
    n_off = np.array(n_off, copy=False, ndmin=1)

    with np.errstate(divide='ignore', invalid='ignore'):
        p_on = n_on / (n_on + n_off)
        p_off = n_off / (n_on + n_off)

        t1 = n_on * np.log(((1 + alpha) / alpha) * p_on)
        t2 = n_off * np.log((1 + alpha) * p_off)

        ts = (t1 + t2)
        significance = np.sqrt(ts * 2)

    significance[np.isnan(significance)] = 0
    significance[n_on < alpha * n_off] = 0

    if scalar:
        return significance[0]

    return significance



from .statistics import li_ma_significance
from .binning import ontime_binning, qla_binning, groupby_observation_blocks, bin_runs

from .core import calc_run_summary_source_independent, split_on_off_source_independent
from .core import calc_run_summary_source_dependent, split_on_off_source_dependent


__all__ = [
    'li_ma_significance',
    'ontime_binning',
    'qla_binning',
    'groupby_observation_blocks',
    'bin_runs',
    'calc_run_summary_source_independent',
    'calc_run_summary_source_dependent',
    'split_on_off_source_dependent',
    'split_on_off_source_independent',
]
